
/*
PREDICATES
  attach_query()
  nondeterm query_argument(string,string)
  clear_query()
  change_query(string,string) % LOP avril 98  
*/
PREDICATES
  cgi_GetString( string )

/******************************************************************************
   Database for dinamic loading name/value query facts
******************************************************************************/
database - query_arguments
  q_arg(
        string ,       % Argument name
        string         % Argument value
       )


/*=============================================================================
                get_query( ECB, ComLine, Arg_String )
  Retrieves the string and command line sent by server
=============================================================================*/
PREDICATES
  retrieve_POST_string(STRING)-(o)
  convert_to_string(INTEGER,BINARY,STRING)-(i,i,o)

CLAUSES
  retrieve_POST_string(CGI_String) :-
        envsymbol("CONTENT_LENGTH",LenStr),
        str_int(LenStr,Len),!,
        readblock(Len,PasteBlock),
	convert_to_string(Len,PasteBlock,CGI_String).
  retrieve_POST_string(_) :-
	errorexit().

  % Creates the string with given length from given binary
  convert_to_string(Len,Bin,Str) :-
        str_len(Str,Len),
        Src=cast(ULONG,Bin),
        Dst=cast(ULONG,Str),
        movmem(Src,Dst,Len).


/*=============================================================================
      Retrieves the query string sent by the server with "GET" request method .
 If request is made with "GET" method the data is placed
 in the "QUERY_STRING" environment variable
=============================================================================*/
PREDICATES
  retrieve_GET_string( string )

CLAUSES
  retrieve_GET_string( CGI_String ) :-
        envsymbol("QUERY_STRING",CGI_String),!.
  retrieve_GET_string("").


/*=============================================================================
                        get_CGI_string(CGI_String)
      Retrieves the query string sent by the server .
=============================================================================*/
PREDICATES
  procedure get_CGI_string_by_method(STRING method,STRING cgiString)

CLAUSES
  get_CGI_string_by_method("POST",CGI_String) :-
        retrieve_POST_string(CGI_String),!.
  get_CGI_string_by_method("GET",CGI_String) :-
        retrieve_GET_string(CGI_String),!.
  get_CGI_string_by_method(_,_) :- 
	errorexit().

  cgi_GetString( CGI_String ) :-
        envsymbol("REQUEST_METHOD",Method),!,
        get_CGI_string_by_method(Method,CGI_String).
  cgi_GetString("").

/*=============================================================================
  decode_urle_value( EncodedString , DecodedString )
  Processes decoding of input URL-encoded string
=============================================================================*/
PREDICATES
  decode_urle_value(string,string)

  decode_urle_symbol(string,char,string)
  decode_urle_symbol(char,string,char,string)
  decode_char(char,integer)

CLAUSES

  decode_urle_value("",""):-!.
  decode_urle_value(CodedString,NormalString) :-
        decode_urle_symbol(CodedString,FrontSymb,CodedTail),
        decode_urle_value(CodedTail,NormalTail),
        frontchar(NormalString,FrontSymb,NormalTail).

  % Retrieves the first decoded symbol from the CGI string
  decode_urle_symbol(Str,Symb,RestStr) :-
        frontchar(Str,C0,Rest0),
        decode_urle_symbol(C0,Rest0,Symb,RestStr).

  decode_urle_symbol('+',Rest,' ',Rest):-!.
  decode_urle_symbol('%',Rest0,Symb,Rest) :-!,
        frontchar(Rest0,C0,Rest1),
        frontchar(Rest1,C1,Rest),
        decode_char(C0,HighQu),
        decode_char(C1,LowQu),
        AsciiCode = HighQu*16 + LowQu,
        char_int(Symb,AsciiCode).
  decode_urle_symbol(Symb,Str,Symb,Str).

  % Primitive char_to_hex convertion
  decode_char(C,HV) :-
        '0' <= C , C <= '9' ,!,
                HV = C - '0' .
  decode_char(C,HV) :-
        'A' <= C , C <= 'F' ,!,
                HV = C - 'A' + 10.
  decode_char(C,HV) :-
        'a' <= C , C <= 'f' ,!,
                HV = C - 'a' + 10.

/*=============================================================================
        nondeterm urle_argument ( Arg_String, Name, Value )
  Sequently scans the input Arg_String for Name/Value pairs
=============================================================================*/
PREDICATES
  nondeterm
  urle_argument( string, string, string )

  urle_arg( string, string, string, string )
  urle_arg_end( string, string, string )

CLAUSES

  urle_argument( ArgS , Name, Value ) :-
        urle_arg( ArgS, Name, Value , _ ).
  urle_argument( ArgS, Name, Value ) :-
        urle_arg( ArgS,_,_,RestS ), urle_argument(RestS,Name,Value).

  urle_arg( ArgS, Name, Value, Rest ) :-
        searchchar(ArgS,'=',EqPos),
        NameLen = EqPos-1 ,
        frontstr(NameLen,ArgS,Name,Rest0),
        frontchar(Rest0,_,Rest1),
        urle_arg_end(Rest1,EncodedValue,Rest),
        decode_urle_value(EncodedValue,Value).

  urle_arg_end(ValStr,Value,Rest) :-
        searchchar(ValStr,'&',ValEndPos),
        ValLen = ValEndPos-1,
        frontstr(ValLen,ValStr,Value,Rest0),
        frontchar(Rest0,_,Rest),
        !.
  urle_arg_end(Value,Value,"").


/*=============================================================================
        load_query_arguments( Arg_String )
 Loads the Name/Value pairs from the input string to the internal database
=============================================================================*/
PREDICATES
   load_query_arguments( string )

CLAUSES
  load_query_arguments( Arg_String ) :-
          urle_argument( Arg_String , Name, Value ),
          assertz(q_arg(Name,Value),query_arguments),
        fail.
  load_query_arguments(_).


/*=============================================================================
        Implementation of main tools predicates
=============================================================================*/

CLAUSES

  attach_query() :-
        retractall(_,query_arguments),
	cgi_GetString(Arg_String),
        load_query_arguments( Arg_String ),
        !.
  attach_query () .


  query_argument( Name, Value ) :-
        q_arg(Name,Value).

  query_argumento( Name, Value ) :-
        q_arg(Name,Value).

  query_argumenti( Name, Value ) :-
        q_arg(Name,Value),!.

  clear_query() :-
                retractall(_,query_arguments).
                
  change_query(NAME,_VALUE) :-
  	retract(q_arg(NAME,_)),
  	fail.
  change_query(NAME,VALUE) :-
	assertz(q_arg(NAME,VALUE)).

  remove_query(NAME) :-
  	retract(q_arg(NAME,_)),!.
  remove_query(_).
  
  test_arg() :-
  	save("test_arg.tst",query_arguments).
